package com.emperador.blue_plugin_example

import CommunicationService
import MESSAGE_ERROR
import MESSAGE_READ
import MESSAGE_WRITE
import android.Manifest
import android.annotation.SuppressLint
import android.app.Activity
import android.bluetooth.*
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.PackageManager
import android.os.Build
import android.os.Handler
import android.os.Looper
import android.util.Log
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import io.flutter.embedding.android.FlutterActivity
import io.flutter.embedding.engine.FlutterEngine
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodChannel
import java.io.IOException

@SuppressLint("MissingPermission")
class MainActivity : FlutterActivity() {

    private var connectThread: ConnectThread? = null
    private val CHANNEL_METHOD_BLUE = "com.emperador.blueplugin/blue"
    private val CHANNEL_STREAM_DEVICES = "com.emperador.blueplugin/devices"
    private val CHANNEL_STREAM_CONNECTION = "com.emperador.blueplugin/connection"

    private var deviceSink: EventChannel.EventSink? = null
    private var connectionSink: EventChannel.EventSink? = null

    private lateinit var adapter: BluetoothAdapter
    private var permissionGranted: Boolean = false
    private var connectedDevice: BluetoothDevice? = null
    private val TAG = "BluePlugin"
    private var _devices: ArrayList<BluetoothDevice> = ArrayList()
    lateinit var bluetoothManager: BluetoothManager

    override fun onStart() {
        super.onStart()
        checkPermission()
        init()
    }

    private fun init() {
        if (!permissionGranted) {
            checkPermission()
            return
        }
        initListeners()
        registerAdapter()

    }

    override fun configureFlutterEngine(flutterEngine: FlutterEngine) {
        super.configureFlutterEngine(flutterEngine)
        MethodChannel(flutterEngine.dartExecutor.binaryMessenger, CHANNEL_METHOD_BLUE).setMethodCallHandler { call, result ->

            when {
                call.method.equals("start-search") -> scanForDevices()
                call.method.equals("stop-search") -> stopScanDevices()
                call.method.equals("connect") -> connect(call.argument<String>("address"))
                call.method.equals("disconnect") -> disconnect()
                call.method.equals("send-trama") -> sendTrama(call.argument<String>("trama")!!)
                else -> {
                    result.notImplemented()
                }
            }
        }


        EventChannel(flutterEngine.dartExecutor.binaryMessenger, CHANNEL_STREAM_DEVICES).setStreamHandler(object : EventChannel.StreamHandler {
            override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
                deviceSink = events
            }

            override fun onCancel(arguments: Any?) {
                deviceSink = null
            }
        })

        EventChannel(flutterEngine.dartExecutor.binaryMessenger, CHANNEL_STREAM_CONNECTION).setStreamHandler(object : EventChannel.StreamHandler {
            override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
                connectionSink = events
            }

            override fun onCancel(arguments: Any?) {
                connectionSink = null
            }
        })
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        checkPermission()
        init()
    }

    private fun connect(address: String?) {
        val device = _devices.find { it.address == address }
        device?.let { connectToDevice(it) }
    }

    private fun disconnect() {
        _commService?.stop()
        _commService = null
        connectThread?.cancel()
        connectThread = null
        connectedDevice = null
        runOnUiThread {
            connectionSink?.success(0)
        }
    }

    private fun registerAdapter() {
        adapter = getBluetoothAdapter()
    }

    private fun initListeners() {
        val filter = IntentFilter(BluetoothDevice.ACTION_FOUND)
        registerReceiver(receiver, filter)
    }

    private fun scanForDevices() {
        getPairedDevices()
        adapter.startDiscovery()
    }

    private fun stopScanDevices() {
        adapter.cancelDiscovery()
    }

    private val receiver = object : BroadcastReceiver() {

        override fun onReceive(context: Context, intent: Intent) {
            when (intent.action!!) {
                BluetoothDevice.ACTION_FOUND -> {
                    val device: BluetoothDevice = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE)!!
                    notifyDeviceFoundOrChange(device)
                }
            }
        }
    }

    private fun parseBluetoothDeviceToHashMap(device: BluetoothDevice): HashMap<String, Any> {
        return hashMapOf(
            "NAME" to device.name,
            "ADDR" to device.address,
            "BOND_STATE" to device.bondState,
            "TYPE" to device.type,
        )
    }

    private fun checkPermission() {
        val permissions = getRequiredPermissions(this)

        for (s in permissions) {
            permissionGranted = ContextCompat.checkSelfPermission(
                activity, s
            ) == PackageManager.PERMISSION_GRANTED
            notifyLogs("$s -> $permissionGranted")
        }

        if (!permissionGranted) {
            ActivityCompat.requestPermissions(activity, permissions, 12)
        }

    }

    private fun getRequiredPermissions(activity: Activity): Array<String> {
        val targetSdkVersion: Int = activity.applicationInfo.targetSdkVersion
        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S && targetSdkVersion >= Build.VERSION_CODES.S) {
            arrayOf(Manifest.permission.BLUETOOTH_SCAN, Manifest.permission.BLUETOOTH_CONNECT)
        } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q && targetSdkVersion >= Build.VERSION_CODES.Q) {
            arrayOf(Manifest.permission.ACCESS_FINE_LOCATION)
        } else arrayOf(Manifest.permission.ACCESS_COARSE_LOCATION)
    }

    private fun getBluetoothAdapter(): BluetoothAdapter {
        bluetoothManager = activity.getSystemService(BluetoothManager::class.java)
        val bluetoothAdapter: BluetoothAdapter? = bluetoothManager.adapter
        if (bluetoothAdapter == null) {
            notifyLogs("Device does not support Bluetooth")
            throw Error("Device does not support Bluetooth")
        }
        notifyLogs("adapter $bluetoothAdapter")
        return bluetoothAdapter
    }

    private fun getPairedDevices(): Set<BluetoothDevice> {
        val pairedDevices: Set<BluetoothDevice> = adapter.bondedDevices
        notifyLogs("cantidad de emparejados : ${pairedDevices.size}")
        pairedDevices.forEach { device ->
            notifyDeviceFoundOrChange(device)
        }
        return pairedDevices
    }

    private fun notifyDeviceFoundOrChange(device: BluetoothDevice) {
        deviceSink?.success(parseBluetoothDeviceToHashMap(device))
        if (!_devices.contains(device))
            _devices.add(device)
    }

    private fun connectToDevice(device: BluetoothDevice) {
        connectThread = ConnectThread(device)
        connectThread?.start()
    }

    private inner class ConnectThread(val device: BluetoothDevice) : Thread() {

        private val mmSocket: BluetoothSocket? by lazy(LazyThreadSafetyMode.NONE) {
            device.createRfcommSocketToServiceRecord(device.uuids[0].uuid)
        }

        override fun run() {
            adapter.cancelDiscovery()
            mmSocket?.let { socket ->
                try {
                    socket.connect()
                    onConnectionEstablished(socket, device)
                } catch (e: Error) {
                    connectedDevice = null
                    connectionSink?.success(0)
                    notifyLogs("${e.message}")
                }
            }
        }

        // Closes the client socket and causes the thread to finish.
        fun cancel() {
            try {
                connectedDevice = null
                mmSocket?.close()
                connectionSink?.success(0)
            } catch (e: IOException) {
                notifyLogs("Could not close the client socket ${e.message}")
            }
        }
    }

    private val handler = Handler(Looper.getMainLooper()) {
        it.apply {
            when (it.what) {
                MESSAGE_ERROR -> Log.e(TAG, "MESSAGE_ERROR")
                MESSAGE_WRITE -> {
                    Log.d(TAG, "MESSAGE_WRITE")
                }
                MESSAGE_READ -> {
                    Log.d(TAG, "MESSAGE_READ")
                    Log.d(TAG, (it.obj as ByteArray).decodeToString(0, it.arg1))
                }
            }
        }
        true
    }

    private var _commService: CommunicationService? = null

    private fun sendTrama(trama: String) {
        notifyLogs("INPUT TEXT -> $trama")
        val byteArray = ArrayList<Byte>()
        val splitTrama = trama.split(",")
        for (i in splitTrama.indices) {
            if (splitTrama[i].toIntOrNull() != null)
                byteArray.add(splitTrama[i].toInt().toByte())
            else
                byteArray.add(splitTrama[i].encodeToByteArray()[0])
        }
        val formatted = byteArray.toByteArray()
        notifyLogs("TO BYTE ARRAY -> $byteArray")
        _commService?.write(formatted)
    }

    private fun onConnectionEstablished(socket: BluetoothSocket, device: BluetoothDevice) {
        connectedDevice = device
        runOnUiThread {
            connectionSink?.success(1)
        }
        _commService = CommunicationService(handler, socket)
        _commService?.start()
    }

    private fun getConnectedDevices() {
        val devices = bluetoothManager.getConnectedDevices(BluetoothProfile.GATT)
    }

    private fun isConnected(device: BluetoothDevice) {
        val devices = bluetoothManager.getConnectionState(device, BluetoothProfile.GATT)
    }

    private fun notifyLogs(text: String) {
        Log.d(TAG, text)
    }

    override fun onDestroy() {
        super.onDestroy()
        unregisterReceiver(receiver)
    }
}
