import 'dart:async';

import 'package:flutter/services.dart';

enum ConnectionDeviceState {
  DISCONNECTED,
  CONNECTED,
}

enum BondState {
  BOND_NONE,
  BOND_BONDING,
  BOND_BONDED,
}

enum DeviceType {
  DEVICE_TYPE_UNKNOWN,
  DEVICE_TYPE_CLASSIC,
  DEVICE_TYPE_LE,
  DEVICE_TYPE_DUAL,
}

class Device {
  final String name;
  final String address;
  final BondState bondState;
  final DeviceType deviceType;

  Device({
    required this.name,
    required this.address,
    required this.bondState,
    required this.deviceType,
  });
}

class BluePlugin {
  static const MethodChannel _mainMethodChannel =
      MethodChannel('com.emperador.blueplugin/blue');

  static const EventChannel _deviceEventChannel =
      EventChannel('com.emperador.blueplugin/devices');

  static const EventChannel _logsEventChannel =
      EventChannel('com.emperador.blueplugin/logs');

  static const EventChannel _connectionEventChannel =
      EventChannel('com.emperador.blueplugin/connection');

  static Stream<Device> _deviceStream = const Stream.empty();
  static Stream<int> _connectionStream = const Stream.empty();
  static Stream<String> _logsStream = const Stream.empty();

  static Stream<String> registerLogsListener() {
    _logsStream = _logsEventChannel.receiveBroadcastStream().map<String>((e) {
      return e;
    });
    return _logsStream;
  }

  static Stream<int> registerDeviceConnectionListener() {
    _connectionStream =
        _connectionEventChannel.receiveBroadcastStream().map<int>((e) {
      return e;
    });
    return _connectionStream;
  }

  static Stream<Device> registerDevicesListener() {
    _deviceStream = _deviceEventChannel.receiveBroadcastStream().map((e) {
      var device = Device(
        name: e["NAME"],
        address: e["ADDR"],
        bondState: _getBondState(e["BOND_STATE"]),
        deviceType: _geDeviceType(e["TYPE"]),
      );
      return device;
    });
    return _deviceStream;
  }

  static BondState _getBondState(int state) {
    switch (state) {
      case 10:
        return BondState.BOND_NONE;
      case 11:
        return BondState.BOND_BONDING;
      case 12:
        return BondState.BOND_BONDED;
      default:
        return BondState.BOND_NONE;
    }
  }

  static DeviceType _geDeviceType(int type) {
    switch (type) {
      case 0:
        return DeviceType.DEVICE_TYPE_UNKNOWN;
      case 1:
        return DeviceType.DEVICE_TYPE_CLASSIC;
      case 2:
        return DeviceType.DEVICE_TYPE_LE;
      case 3:
        return DeviceType.DEVICE_TYPE_DUAL;
      default:
        return DeviceType.DEVICE_TYPE_UNKNOWN;
    }
  }

  static startSearch() => _mainMethodChannel.invokeMethod('start-search');

  static stopSearch() => _mainMethodChannel.invokeMethod('stop-search');

  static disconnectDevice() => _mainMethodChannel.invokeMethod('disconnect');

  static connect(String address) => _mainMethodChannel.invokeMethod(
        'connect',
        {'address': address},
      );

  static sendTrama(String trama) => _mainMethodChannel.invokeMethod(
        'send-trama',
        {'trama': trama},
      );
}
